--includes
	include graphics.e
	include wildcard.e
	include Rtcgen.e
	include Rslice.e
	include Rstrings.e
		--Rslice.e
	include Rwrite.e
		--Rstrings.e

	with trace

integer
	KeyFound,
	KeyPos,
	SecFound,
	SecBeg,
	SecEnd

object
	Rtn

sequence
	Cmd,
	FileSpec,
	IniFilBuf,
	IniFilSpc,
	Key,
	NotFoundMsg,
	Section,
	Value,
	ValRead

--===========================================================================--
procedure Initialize()
	NotFoundMsg = "NOTFOUND"
	FileSpec = ""
	Cmd = ""
	Section = ""
	Key = ""
	Value = ""
	IniFilBuf = {}
	KeyFound = FALSE
	KeyPos = 0
	SecFound = FALSE
	SecBeg = 0
	SecEnd = 0
end procedure --

procedure ParseCmdLin(sequence CmdLin)
	FileSpec = CmdLin[1]
	Cmd = upper(CmdLin[2])
	Section = CmdLin[3]
	Key = CmdLin[4]
	Value = CmdLin[5]
end procedure --ParseCmdLin

--===========================================================================--
function GetIniFilSpc(object FileSpec) -->seq
 --If the file spec passed on the command line is a valid file, use it.
	if FileExists(FileSpec) then
		return FileSpec
	end if
 --If file spec not a valid file, see if it is env var holding a valid file
	FileSpec = getenv(FileSpec)
	if not atom(FileSpec) then
		--If there is such an env var, see it it is a file
		if FileExists(FileSpec) then
			return FileSpec
		end if
	end if
	return "" --No valid file found
end function --GetIniFilSpc

procedure SetFileSpec()
	IniFilSpc = GetIniFilSpc(FileSpec)
	if not String(IniFilSpc) then
		ErrAbort("Bad ini file spec "&IniFilSpc,1)
	end if
	if compare(IniFilSpc,"") = 0 then
		ErrAbort("Could not locate "&FileSpec,1)
	end if
end procedure --SetFileSpec
-------------------------------------------------------------------------------
function GetKey(sequence Buf)
	return TrimSpace(upper(LeftToChar(Buf,'=')))
end function --GetKey

function GetVal(sequence Buf) --> seq
	return TrimSpace(RightToChar(Buf,'='))
end function --GetVal

procedure FindKeyInSection()
	--If the key is found in the found section, then the KeyFound flag will be set and
	--the KeyPos will be set
	sequence
		CurLin,
		uKey
	uKey = upper(Key)
	for i=SecBeg to SecEnd do
		CurLin = upper(IniFilBuf[i])
		if compare(uKey,GetKey(IniFilBuf[i])) = 0 then
			KeyFound = TRUE
			KeyPos = i
			return
		end if
	end for
end procedure --FindKeyInSection

procedure FindSecBegEnd()
	--If the section is found, then SecFound flag will be set and
	--SecBeg and SecEnd will be set
	sequence
		CurLin,
		uSec
	uSec = upper(Section)
	for i=1 to length(IniFilBuf) do
		--Trim all lines
		IniFilBuf[i] = TrimSpace(IniFilBuf[i])
		--Trim section headings in []
		if IniFilBuf[i][1]='[' then
			IniFilBuf[i]='['&TrimSpace(sSub(IniFilBuf[i],2,length(IniFilBuf[i])-2))&"]\n"
		end if
	end for
	for i=1 to length(IniFilBuf) do
		CurLin = IniFilBuf[i]
		--Is cur line a section line
		if CurLin[1]='[' then
			if SecFound then
				--If already in my target section, then a new section line is the end
				SecEnd = i-1
				return
			end if
			CurLin = sSub(CurLin,2,length(CurLin)-2)
			CurLin = upper(TrimSpace(CurLin))
			--If this is the section I am looking for, flag it
			if compare(CurLin,uSec) = 0 then
				SecFound = TRUE
				SecBeg = i
			end if
		end if
	end for
	--If I finished the loop in the target section, then I came to the end of the file
	--and the end of the section is the end of the file
	if SecFound then SecEnd = length(IniFilBuf) end if
end procedure --FindSecBegEnd

-------------------------------------------------------------------------------
procedure ProcessCmd()
	--Delete
	if compare(Cmd,"DELETE") = 0 then
		if KeyFound then
			IniFilBuf = Delete(IniFilBuf,KeyPos)
		end if
		Rtn = PutTextFile(IniFilSpc,IniFilBuf)
	end if
	--Read
	if compare(Cmd,"READ") = 0 then
		if KeyFound then
			ValRead = GetVal(IniFilBuf[KeyPos])
			puts(STDOUT,ValRead)
		else
			puts(STDOUT,NotFoundMsg)
		end if
	end if
	--Write
	if compare(Cmd,"WRITE") = 0 then
		if SecFound then
			if KeyFound then
				IniFilBuf[KeyPos] = Key&'='&Value&'\n'
			else
				while TRUE do
					if compare(TrimAll(IniFilBuf[SecEnd]),"")=0 then
						--If adding at end of section, go before ending blank lines
						SecEnd = SecEnd - 1
					else
						exit
					end if
				end while
				IniFilBuf = Insert(IniFilBuf,SecEnd,Key&'='&Value&'\n')
			end if
		else
			IniFilBuf = append(IniFilBuf,'['&Section&"]\n")
			IniFilBuf = append(IniFilBuf,Key&'='&Value&'\n')
		end if
		Rtn = PutTextFile(IniFilSpc,IniFilBuf)
	end if
end procedure --ProcessCmd

--===========================================================================--
global procedure IniFile(sequence CmdLin)
	--Initialize
	Initialize()
	ParseCmdLin(CmdLin)
	--Get/fix the ini file spec and file
	SetFileSpec()
	--Get the ini file text into a seq
	IniFilBuf = GetTextFile(IniFilSpc)
	if not OkayGlb then
		ErrAbort("Could not open "&IniFilSpc,1)
	end if
	--Find the begin and end pos of the section
	FindSecBegEnd()
	--Get find the key. Set KeyPos
	if SecFound then FindKeyInSection() end if
	ProcessCmd()
end procedure --IniFile